from typing import Final

BOARD: Final = 10
BCM: Final = 11
TEGRA_SOC: Final = 1000
CVM: Final = 1001

PUD_OFF: Final = 20
PUD_DOWN: Final = 21
PUD_UP: Final = 22

HIGH: Final = 1
LOW: Final = 0

RISING: Final = 31
FALLING: Final = 32
BOTH: Final = 33

UNKNOWN: Final = -1
OUT: Final = 0
IN: Final = 1
HARD_PWM: Final = 43
