from authlib.oauth2.rfc6749 import TokenValidator

class IntrospectTokenValidator(TokenValidator):
    TOKEN_TYPE: str
    def introspect_token(self, token_string): ...
    def authenticate_token(self, token_string): ...
    def validate_token(self, token, scopes, request) -> None: ...
