from functools import cached_property

from authlib.oauth2.rfc6749 import JsonPayload, JsonRequest, OAuth2Payload, OAuth2Request

class FlaskOAuth2Payload(OAuth2Payload):
    def __init__(self, request) -> None: ...
    @property
    def data(self): ...
    @cached_property
    def datalist(self): ...

class FlaskOAuth2Request(OAuth2Request):
    payload: FlaskOAuth2Payload
    def __init__(self, request) -> None: ...
    @property
    def args(self): ...
    @property
    def form(self): ...

class FlaskJsonPayload(JsonPayload):
    def __init__(self, request) -> None: ...
    @property
    def data(self): ...

class FlaskJsonRequest(JsonRequest):
    payload: FlaskJsonPayload
    def __init__(self, request) -> None: ...
