class ArcIV:
    def __init__(self, key) -> None: ...
    def reset(self) -> None: ...
    def encode(self, S: str | bytes | list[int]) -> bytes: ...

def encode(text: str | bytes | list[int], key) -> bytes: ...
def decode(text: str | bytes | list[int], key) -> bytes: ...

__all__ = ["ArcIV", "encode", "decode"]
