from typing import SupportsIndex

def comb(n: SupportsIndex, k: SupportsIndex, /) -> int: ...
def factorial(n: SupportsIndex, /) -> int: ...
def gcd(*integers: SupportsIndex) -> int: ...
def isqrt(n: SupportsIndex, /) -> int: ...
def lcm(*integers: SupportsIndex) -> int: ...
def perm(n: SupportsIndex, k: SupportsIndex | None = None, /) -> int: ...
