from authlib.oauth2.rfc6749 import JsonPayload, JsonRequest, OAuth2Payload, OAuth2Request

class DjangoOAuth2Payload(OAuth2Payload):
    def __init__(self, request) -> None: ...

class DjangoOAuth2Request(OAuth2Request):
    payload: DjangoOAuth2Payload
    def __init__(self, request) -> None: ...
    @property
    def args(self): ...
    @property
    def form(self): ...

class DjangoJsonPayload(JsonPayload):
    def __init__(self, request) -> None: ...
    def data(self): ...

class DjangoJsonRequest(JsonRequest):
    payload: DjangoJsonPayload
    def __init__(self, request) -> None: ...
