from datetime import datetime, tzinfo

from google.protobuf.timestamp_pb2 import Timestamp

def from_json_string(value: str) -> Timestamp: ...
def from_microseconds(micros: float) -> Timestamp: ...
def from_milliseconds(millis: float) -> Timestamp: ...
def from_nanoseconds(nanos: float) -> Timestamp: ...
def from_seconds(seconds: float) -> Timestamp: ...
def from_current_time() -> Timestamp: ...
def to_json_string(ts: Timestamp) -> str: ...
def to_microseconds(ts: Timestamp) -> int: ...
def to_milliseconds(ts: Timestamp) -> int: ...
def to_nanoseconds(ts: Timestamp) -> int: ...
def to_seconds(ts: Timestamp) -> int: ...
def to_datetime(ts: Timestamp, tz: tzinfo | None = None) -> datetime: ...
