from _typeshed import Incomplete
from typing import NamedTuple

class _Event(NamedTuple):
    when: Incomplete
    callback: Incomplete
    args: Incomplete
    kwargs: Incomplete

class EventLoop:
    current: Incomplete
    idlers: Incomplete
    inactive: int
    queue: Incomplete
    rpcs: Incomplete
    rpc_results: Incomplete
    def __init__(self) -> None: ...
    def clear(self) -> None: ...
    def insort_event_right(self, event) -> None: ...
    def call_soon(self, callback, *args, **kwargs) -> None: ...
    def queue_call(self, delay, callback, *args, **kwargs) -> None: ...
    def queue_rpc(self, rpc, callback) -> None: ...
    def add_idle(self, callback, *args, **kwargs) -> None: ...
    def run_idle(self): ...
    def run0(self): ...
    def run1(self): ...
    def run(self) -> None: ...
