from authlib.oauth2.rfc7009 import RevocationEndpoint as _RevocationEndpoint

class RevocationEndpoint(_RevocationEndpoint):
    def query_token(self, token, token_type_hint): ...
    def revoke_token(self, token, request) -> None: ...
