from _typeshed import FileDescriptorOrPath
from typing import Final

__version__: Final[str]
PLAIN: Final = 1
PREFORMATTED: Final = 2
BULLETCHAR: Final = "\267"

class BaseParser:
    def __init__(self) -> None: ...
    def reset(self) -> None: ...
    def parseFile(self, filename: FileDescriptorOrPath) -> list[tuple[str, str] | tuple[str, str, str]]: ...
    def parseText(self, textBlock: str) -> list[tuple[str, str] | tuple[str, str, str]]: ...
    def readLine(self, line: str) -> None: ...
    def endPara(self) -> None: ...
    def beginPre(self, stylename: str) -> None: ...
    def endPre(self) -> None: ...
    def image(self, filename: str) -> None: ...

class Parser(BaseParser):
    def vSpace(self, points) -> None: ...
    def pageBreak(self) -> None: ...
    def custom(self, moduleName: str, funcName: str) -> None: ...
    def nextPageTemplate(self, templateName: str) -> None: ...

def parseFile(filename: FileDescriptorOrPath) -> list[tuple[str, str] | tuple[str, str, str]]: ...
def parseText(textBlock: str) -> list[tuple[str, str] | tuple[str, str, str]]: ...
