from _typeshed import Incomplete

def add_to_uri(token, uri) -> str: ...
def add_to_headers(token, headers=None): ...
def add_to_body(token, body=None) -> str: ...
def add_bearer_token(token, uri, headers, body, placement: str = "header") -> tuple[Incomplete, Incomplete, Incomplete]: ...
