from typing import Final

class TransactionAmounts:
    Authorize: Final = "1000.00"
    PartiallyAuthorized: Final = "1004.00"
    Decline: Final = "2000.00"
    HardDecline: Final = "2015.00"
    Fail: Final = "3000.00"
