from types import ModuleType
from typing import Any

from google.protobuf.descriptor import FileDescriptor

def BuildMessageAndEnumDescriptors(file_des: FileDescriptor, module: dict[str, Any]) -> None: ...
def BuildTopDescriptorsAndMessages(file_des: FileDescriptor, module_name: str, module: dict[str, Any]) -> None: ...
def AddHelpersToExtensions(file_des: FileDescriptor) -> None: ...
def BuildServices(file_des: FileDescriptor, module_name: str, module: ModuleType) -> None: ...
