from typing import Final

STGC_DEFAULT: Final = 0
STGC_OVERWRITE: Final = 1
STGC_ONLYIFCURRENT: Final = 2
STGC_DANGEROUSLYCOMMITMERELYTODISKCACHE: Final = 4
STGC_CONSOLIDATE: Final = 8
STGTY_STORAGE: Final = 1
STGTY_STREAM: Final = 2
STGTY_LOCKBYTES: Final = 3
STGTY_PROPERTY: Final = 4
STREAM_SEEK_SET: Final = 0
STREAM_SEEK_CUR: Final = 1
STREAM_SEEK_END: Final = 2
LOCK_WRITE: Final = 1
LOCK_EXCLUSIVE: Final = 2
LOCK_ONLYONCE: Final = 4
CWCSTORAGENAME: Final = 32
STGM_DIRECT: Final = 0x00000000
STGM_TRANSACTED: Final = 0x00010000
STGM_SIMPLE: Final = 0x08000000
STGM_READ: Final = 0x00000000
STGM_WRITE: Final = 0x00000001
STGM_READWRITE: Final = 0x00000002
STGM_SHARE_DENY_NONE: Final = 0x00000040
STGM_SHARE_DENY_READ: Final = 0x00000030
STGM_SHARE_DENY_WRITE: Final = 0x00000020
STGM_SHARE_EXCLUSIVE: Final = 0x00000010
STGM_PRIORITY: Final = 0x00040000
STGM_DELETEONRELEASE: Final = 0x04000000
STGM_NOSCRATCH: Final = 0x00100000
STGM_CREATE: Final = 0x00001000
STGM_CONVERT: Final = 0x00020000
STGM_FAILIFTHERE: Final = 0x00000000
STGM_NOSNAPSHOT: Final = 0x00200000
ASYNC_MODE_COMPATIBILITY: Final = 0x00000001
ASYNC_MODE_DEFAULT: Final = 0x00000000
STGTY_REPEAT: Final = 0x00000100
STG_TOEND: Final = 0xFFFFFFFF
STG_LAYOUT_SEQUENTIAL: Final = 0x00000000
STG_LAYOUT_INTERLEAVED: Final = 0x00000001
COM_RIGHTS_EXECUTE: Final = 1
COM_RIGHTS_EXECUTE_LOCAL: Final = 2
COM_RIGHTS_EXECUTE_REMOTE: Final = 4
COM_RIGHTS_ACTIVATE_LOCAL: Final = 8
COM_RIGHTS_ACTIVATE_REMOTE: Final = 16
STGFMT_DOCUMENT: Final = 0
STGFMT_STORAGE: Final = 0
STGFMT_NATIVE: Final = 1
STGFMT_FILE: Final = 3
STGFMT_ANY: Final = 4
STGFMT_DOCFILE: Final = 5
PID_DICTIONARY: Final = 0
PID_CODEPAGE: Final = 1
PID_FIRST_USABLE: Final = 2
PID_FIRST_NAME_DEFAULT: Final = 4095
PID_LOCALE: Final = -2147483648
PID_MODIFY_TIME: Final = -2147483647
PID_SECURITY: Final = -2147483646
PID_BEHAVIOR: Final = -2147483645
PID_ILLEGAL: Final = -1
PID_MIN_READONLY: Final = -2147483648
PID_MAX_READONLY: Final = -1073741825
PIDDI_THUMBNAIL: Final = 0x00000002
PIDSI_TITLE: Final = 2
PIDSI_SUBJECT: Final = 3
PIDSI_AUTHOR: Final = 4
PIDSI_KEYWORDS: Final = 5
PIDSI_COMMENTS: Final = 6
PIDSI_TEMPLATE: Final = 7
PIDSI_LASTAUTHOR: Final = 8
PIDSI_REVNUMBER: Final = 9
PIDSI_EDITTIME: Final = 10
PIDSI_LASTPRINTED: Final = 11
PIDSI_CREATE_DTM: Final = 12
PIDSI_LASTSAVE_DTM: Final = 13
PIDSI_PAGECOUNT: Final = 14
PIDSI_WORDCOUNT: Final = 15
PIDSI_CHARCOUNT: Final = 16
PIDSI_THUMBNAIL: Final = 17
PIDSI_APPNAME: Final = 18
PIDSI_DOC_SECURITY: Final = 19
PIDDSI_CATEGORY: Final = 2
PIDDSI_PRESFORMAT: Final = 3
PIDDSI_BYTECOUNT: Final = 4
PIDDSI_LINECOUNT: Final = 5
PIDDSI_PARCOUNT: Final = 6
PIDDSI_SLIDECOUNT: Final = 7
PIDDSI_NOTECOUNT: Final = 8
PIDDSI_HIDDENCOUNT: Final = 9
PIDDSI_MMCLIPCOUNT: Final = 10
PIDDSI_SCALE: Final = 11
PIDDSI_HEADINGPAIR: Final = 12
PIDDSI_DOCPARTS: Final = 13
PIDDSI_MANAGER: Final = 14
PIDDSI_COMPANY: Final = 15
PIDDSI_LINKSDIRTY: Final = 16
PIDMSI_EDITOR: Final = 2
PIDMSI_SUPPLIER: Final = 3
PIDMSI_SOURCE: Final = 4
PIDMSI_SEQUENCE_NO: Final = 5
PIDMSI_PROJECT: Final = 6
PIDMSI_STATUS: Final = 7
PIDMSI_OWNER: Final = 8
PIDMSI_RATING: Final = 9
PIDMSI_PRODUCTION: Final = 10
PIDMSI_COPYRIGHT: Final = 11
PROPSETFLAG_DEFAULT: Final = 0
PROPSETFLAG_NONSIMPLE: Final = 1
PROPSETFLAG_ANSI: Final = 2
PROPSETFLAG_UNBUFFERED: Final = 4
PROPSETFLAG_CASE_SENSITIVE: Final = 8
STGMOVE_MOVE: Final = 0
STGMOVE_COPY: Final = 1
STGMOVE_SHALLOWCOPY: Final = 2
